#include "Pool.hxx"

#include "IsotopicVector.hxx"
#include "Storage.hxx"
#include "Scenario.hxx"
#include "CLASSLogger.hxx"

#include <sstream>
#include <string>
#include <iostream>
#include <cmath>
#include <algorithm>

//________________________________________________________________________
//
//		Pool
//
//
//
//
//________________________________________________________________________
ClassImp(Pool)


Pool::Pool():CLASSBackEnd(new CLASSLogger("Pool.log"), 8)
{
	fOutBackEndFacility = 0;
	SetName("P_Pool.");
}

	//________________________________________________________________________
Pool::Pool(CLASSLogger* log, cSecond coolingtime):CLASSBackEnd(log, coolingtime, 8)
{


	fCycleTime = (cSecond)coolingtime;
	fIsStarted = false;
	fPutToWaste = true;
	fCoolingLastIndex = 0;

	fOutBackEndFacility = 0;
	SetName("P_Pool.");

	
	INFO	<< " A new Pool has been define :" << endl;
	INFO	<< "\t Creation time set at \t " << (double)(GetCreationTime()/3600/24/365.25) << " year" << endl;
	INFO	<< "\t The Cooling Time set at\t " << (double)(fCycleTime/3600/24/365.25) << " year" << endl;
	WARNING	<< " All Cooled Fuel goes directly to WASTE after cooling !! " << endl;


}

//________________________________________________________________________
Pool::Pool(CLASSLogger* log, CLASSBackEnd* storage, cSecond coolingtime):CLASSBackEnd(log, coolingtime, 8)
{

	fOutBackEndFacility = storage;
	SetIsStorageType(false);

	fIsStarted = false;
	fPutToWaste = false;
	fCoolingLastIndex = 0;
	SetName("P_Pool.");

	

	INFO	<< " A new Pool has been define :" << endl;
	INFO	<< "\t Creation time set at \t " << (double)(GetCreationTime()/3600/24/365.25) << " year" << endl;
	INFO	<< "\t The Cooling Time set at\t " << (double)(fCycleTime/3600/24/365.25) << " year" << endl;


}

//________________________________________________________________________
Pool::~Pool()
{


}

//________________________________________________________________________
//________________________________________________________________________
void Pool::SetIVArray(vector<IsotopicVector> ivarray)
{
	INFO << "This method as no effect !!!" << endl;
	INFO << "Use SetIVArray(vector<IsotopicVector> ivarray, vector<cSecond>n timearray) unstead!!!!"<<endl;
}


//________________________________________________________________________
void Pool::SetIVArray(vector<IsotopicVector> ivarray, vector<cSecond> timearray)
{
	fIVArray = ivarray;
	fIVArrayArrivalTime =  timearray;

}
//________________________________________________________________________
//	Add Temporary IV : 
//		Cooling
//		
//________________________________________________________________________
void Pool::AddIV(IsotopicVector IV)
{ 

	fIVArray.push_back(IV);
	fInsideIV += IV;
	fIVArrayArrivalTime.push_back(fInternalTime);
	fCoolingLastIndex++;
	fCoolingIndex.push_back(fCoolingLastIndex);

	AddCumulativeIVIn(IV);

}


//________________________________________________________________________
void Pool::RemoveIVCooling(int i)		//!< Remove a Cooling IsotopicVector
{
	AddCumulativeIVOut(fIVArray[i]);

	fIVArray.erase(fIVArray.begin()+i);
	fIVArrayArrivalTime.erase( fIVArrayArrivalTime.begin()+i);
	fCoolingIndex.erase(fCoolingIndex.begin()+i); 

}




//________________________________________________________________________
//	Time Action with the reste of the Universe : 
//		Out Storage
//		Evolution : 
//			Cooling
//________________________________________________________________________




//________________________________________________________________________
void Pool::CoolingEvolution(cSecond t)
{
DBGL

	if(t == fInternalTime && t!=0) return;
	int RemainingCoolingTime;
	cSecond EvolutionTime = t - fInternalTime;

	fInsideIV = IsotopicVector();

#pragma omp parallel for
	for ( int i = 0 ; i < (int)fIVArray.size() ; i++)
	{
		if ( abs(t -  fIVArrayArrivalTime[i] - fCycleTime) < 3600 ) // ">" should not append, only "=" is normal...
		{
			if (t -  fIVArrayArrivalTime[i] > fCycleTime) // Warning & Quit
			{
				ERROR << " Cooling Step : " << t/3600./24/365.25<< " :" << " an evolution Step is probably missing ! " << " " << endl;
				exit (1);
			}
   
			RemainingCoolingTime = fCycleTime - (fInternalTime -  fIVArrayArrivalTime[i]);
			//Cooling Decay
			fIVArray[i] = GetDecay( fIVArray[i], RemainingCoolingTime);


#pragma omp critical(DeleteCoolingIVPB)
			{fCoolingEndOfCycle.push_back(i);}

		}
		else if (  fIVArrayArrivalTime[i] != t )
		{
			fIVArray[i] = GetDecay( fIVArray[i] , EvolutionTime);
			fInsideIV += fIVArray[i];
		}
	}
#pragma omp critical(DeleteCoolingIVPB)
	{sort (fCoolingEndOfCycle.begin(), fCoolingEndOfCycle.end());}

DBGL
}


//________________________________________________________________________
void Pool::Evolution(cSecond t)
{

	// Check if the Pool has been created ...
	if(t<GetCreationTime()) return;
	if(t == fInternalTime && t!=0) return;
	
	if(fInternalTime == 0 && !fIsStarted)
	{
		fInternalTime = GetCreationTime();
		fIsStarted = true;
	}

	// Make the evolution for the Cooling IV ...
	CoolingEvolution(t);
	
	// ... And Finaly update the AbsoluteInternalTime
	fInternalTime = t;
	


}


//________________________________________________________________________
void Pool::Dump()
{
DBGL
//------ Cooling ------//
	for(int i = (int)fCoolingEndOfCycle.size()-1; i >=0 ; i--)	// IV End Of Cooling
	{
	
		int idx = fCoolingEndOfCycle[i];			// Get Index number
		
		if(!fPutToWaste)
			fOutBackEndFacility->AddIV(fIVArray[idx]);
		else
			GetParc()->AddWaste(fIVArray[idx]);

		fCoolingEndOfCycle.erase(fCoolingEndOfCycle.begin()+i);	// Remove index entry
		RemoveIVCooling(idx);					// Remove IVcooling
	}
	
	if((int)fCoolingEndOfCycle.size() != 0 )// Control
	{
		ERROR << "Problem while Dumping Cooling"<< endl;
		exit (1);
	}
DBGL
}


